#coding: utf-8
"""
 Copied exactly from https://github.com/hmarr/mongoengine/blob/master/mongoengine/django/tests.py

"""

from django.test import TestCase
from django.conf import settings

from mongoengine import connect

class MongoTestCase(TestCase):
    """
    TestCase class that clear the collection between the tests
    """
    db_name = 'test_%s' % settings.MONGO_DATABASE_NAME
    def __init__(self, methodName='runtest'):
        self.db = connect(self.db_name)
        super(MongoTestCase, self).__init__(methodName)

    def _post_teardown(self):
        super(MongoTestCase, self)._post_teardown()
        for collection in self.db.collection_names():
            if collection == 'system.indexes':
                continue
            self.db.drop_collection(collection)


import logging

from django import forms

from mongoengine.fields import Document, EmbeddedDocumentField, ListField, ReferenceField
from mongonaut.widgets import get_widget

logger = logging.getLogger('mongonaut.forms')

class DocumentListForm(forms.Form):
    """ The main document list form """
    mongo_id = forms.MultipleChoiceField(required=True, widget=forms.CheckboxSelectMultiple)


class DocumentDetailForm(forms.Form):
    pass
    
CHECK_ATTRS = dict(
        required='required',
        help_text='help_text',
        name='name'
    )

def get_document_unicode(document):
    try:
        return document.__unicode__()
    except AttributeError:
        return unicode(document)

def document_detail_form_factory(form, document_type, initial=False):
    """ Adds document field to a form. """    
    for key in document_type._fields.keys():
        field = document_type._fields[key]
        logger.debug(field.__dict__)
        logging.debug('help')
        widget = get_widget(field)
        if isinstance(widget, forms.widgets.Select):
            form.fields[key] = forms.ChoiceField(
                key, 
                required=field.required,
                widget=widget)            
        else:
            form.fields[key] = forms.CharField(
                key, 
                required=field.required,
                widget=widget)
        if initial:

            field_initial =   getattr(initial, key)
            if isinstance(field_initial, Document):
                # probably a reference field so we add some choices
                # TODO - does this actually work? Need tests and test coverage!!!
                form.fields[key].initial = field_initial.id
                form.fields[key].choices = [(unicode(x.id), get_document_unicode(x)) for x in type(field_initial).objects.all()]
            else:
                form.fields[key].initial = field_initial

        if form.fields[key].initial is None and isinstance(field, ReferenceField):
            form.fields[key].choices = [(unicode(x.id), get_document_unicode(x)) for x in field.document_type.objects.all()]
            
        
        for field_key, form_attr in CHECK_ATTRS.items():
            if hasattr(field, field_key):
                value = getattr(field, field_key)
                setattr(form.fields[key], field_key, value)

        # used as a handy reference field
        form.fields[key].mongofield = field
            
    return form





from django.conf.urls.defaults import patterns, include, url

#import mongonaut
#mongonaut.autodiscover()

urlpatterns = patterns('',
    # Examples:
    # url(r'^$', 'blog.views.home', name='home'),
    # url(r'^blog/', include('blog.foo.urls')),

    # Uncomment the admin/doc line below to enable admin documentation:
    # url(r'^admin/doc/', include('django.contrib.admindocs.urls')),

    # Uncomment the next line to enable the admin:
    url(r'^mongonaut/', include('mongonaut.urls')),
)



from django.conf.urls.defaults import patterns, url
from django.views.generic.base import TemplateView
from django.views.generic import ListView

from mongonaut import views

urlpatterns = patterns('',
    url(
        regex=r'^$',
        view=views.IndexView.as_view(),
        name="index"
    ),
    url(
        regex=r'^(?P<app_label>[_\-\w]+)/(?P<document_name>[_\-\w]+)/$',
        view=views.DocumentListView.as_view(),
        name="document_list"
    ),
    url(
        regex=r'^(?P<app_label>[_\-\w]+)/(?P<document_name>[_\-\w]+)/(?P<id>[\w]{24})/$',
        view=views.DocumentDetailView.as_view(),
        name="document_detail"
    ),
    url(
        regex=r'^(?P<app_label>[_\-\w]+)/(?P<document_name>[_\-\w]+)/(?P<id>[\w]{24})/edit/$',
        view=views.DocumentDetailEditFormView.as_view(),
        name="document_detail_edit_form"
    ),    
    url(
        regex=r'^(?P<app_label>[_\-\w]+)/(?P<document_name>[_\-\w]+)/add/$',
        view=views.DocumentDetailAddFormView.as_view(),
        name="document_detail_add_form"
    ),
    url(
        regex=r'^(?P<app_label>[_\-\w]+)/(?P<document_name>[_\-\w]+)/(?P<id>[\w]{24})/delete/$',
        view=views.DocumentDeleteView.as_view(),
        name="document_delete"
    )    
)





# -*- coding: utf-8 -*-
from django import template
from django.core.urlresolvers import reverse
from django.forms.formsets import BaseFormSet
from django.template import Context
from django.template.loader import get_template
from django.utils.safestring import mark_safe

from bson.objectid import ObjectId
from mongoengine import Document

register = template.Library()

@register.simple_tag()
def get_document_value(document, key):
    value = getattr(document, key)
    if isinstance(value, ObjectId):
        return value
        
    if isinstance(value, Document):
        app_label = value.__module__.replace(".models","")
        document_name = value._class_name
        url = reverse("document_detail", kwargs={'app_label':app_label, 'document_name':document_name, 'id':value.id})
        return mark_safe("""<a href="{0}">{1}</a>""".format(url, value.__unicode__()))
        
    return value


from mongonaut.sites import MongoAdmin

from articles.models import Post, User

class PostAdmin(MongoAdmin):
    
    def has_view_permission(self, request):
        return True

    def has_edit_permission(self, request):
        return True

    def has_add_permission(self, request):
        return True

    def has_delete_permission(self, request):
        return True

    search_fields = ('title','id')
    list_fields = ('title', "published", "pub_date")    
    
class UserAdmin(MongoAdmin):    
    def has_view_permission(self, request):
        return True

    def has_edit_permission(self, request):
        return True

    def has_add_permission(self, request):
        return True
    
    list_fields = ('first_name', "last_name", "email")


Post.mongoadmin = PostAdmin()
User.mongoadmin = UserAdmin()